from god.options import options

from renderPageTemplate import renderPageTemplate, cached_XML_Objectify

from Mixin import TableMixin
from activityMixin import ActivityMixin
from Mixin import XMLMixin

import os

class DatabaseGeneration:
        def __init__(self, xmlConfigFileName):
                if xmlConfigFileName:
                        self.initialize(xmlConfigFileName)

        def initialize(self, xmlConfigFileName):
                self.schema = cached_XML_Objectify(xmlConfigFileName).make_instance()
                self.sql_connect = self.schema.tools.database.sql_connect.PCDATA.encode('ascii')
                self.zope_url = self.schema.tools.zope.host.url.encode('ascii')
                self.zope_db_connection = self.schema.tools.zope.database_connection.id.encode('ascii')
                self.directoryPaths = {}
                for directory in self.schema.directorys.directory:
                        dirId = directory.id.encode('ascii')
                        directoryPath = directory.path.encode('ascii')
                        if os.path.isabs(directoryPath) and not os.path.exists(directoryPath):
                                        raise 'directory id [%s] path [%s] does not exists !' % (dirId, directoryPath)
                        self.directoryPaths[directory.id.encode('ascii')] = directoryPath
                self.fileNames = {}
                for file in self.schema.files.file:
                        directoryId = file.directoryId.encode('ascii')
                        fileName = file.name.encode('ascii')
                        fullPathName = self.createFileName(directoryId, fileName)
                        fileId = file.id.encode('ascii')
                        if file.type.encode('ascii') == 'input':
                                if not os.path.exists(fullPathName):
                                        raise 'file id [%s] name [%s] does not exists !' % (fileId, fullPathName)
                        self.fileNames[fileId]= fullPathName

        def getDirectoryPath(self, directoryId):
                try:
                        return self.createDirectoryPath(\
                        self.directoryPaths[directoryId])
                except KeyError:
                        raise 'Directory with Id "%s" not defined !' % directoryId

        def createDirectoryPath(self, directoryPath):
                if os.path.isabs(directoryPath):
                        result = directoryPath
                else:
                        result = self.getRootDirectory() + \
                                     os.sep + directoryPath
                if not os.path.exists(result):
                        os.makedirs(result)
                return result

        def getRootDirectory(self):
                try:
                        return self.directoryPaths['root']
                except KeyError:
                        return os.getcwd()

        def createFileName(self, directoryId, fileName):
                return self.getDirectoryPath(directoryId) \
                                 + os.sep + fileName

        def getFileName(self, fileId):
                try:
                        return self.fileNames[fileId]
                except KeyError:
                        raise 'File with Id "%s" not defined !' % fileId

        def addTracingXML(self, tableSchemaFileName, XMLOutputFileName):
                rpt = renderPageTemplate()
                templateFileName = self.getFileName("allFieldsXML")
                rpt.setTemplateFromFile(templateFileName)
                xmlTracingFieldsFileName =  self.getFileName("tracingXML")
                rpt.addToContext({'tracing':open(xmlTracingFieldsFileName).read()})
                macroFileName =  self.getFileName("xmlMacros")
                rpt.addPageTemplateToContext('macro_pt', macroFileName)
                rpt.addXMLObjectToContext(tableSchemaFileName, {'table':TableMixin})
                rpt.renderToFile(XMLOutputFileName)

        def createNomenCodesXML(self, nomenName, codesXMLOutputFileName):
                #_CODES
                #create XML schema input for table
                nomenName = nomenName.upper()
                rpt = renderPageTemplate()
                templateFileName = self.getFileName('nomenCodes')
                rpt.setTemplateFromFile(templateFileName)
                rpt.addToContext({'nomenName':nomenName})
                macroFileName = self.getFileName('xmlMacros')
                rpt.addPageTemplateToContext('macro_pt', macroFileName)
                rpt.renderToFile(codesXMLOutputFileName)

        def createNomenIntitulesXML(self, nomenName, intitulesXMLOutputFileName):
                #_INTITULES        
                #create XML schema input for table
                nomenName = nomenName.upper()
                rpt = renderPageTemplate()
                templateFileName = self.getFileName('nomenIntitules')
                rpt.setTemplateFromFile(templateFileName)
                rpt.addToContext({'nomenName':nomenName})
                macroFileName = self.getFileName('xmlMacros')
                rpt.addPageTemplateToContext('macro_pt', macroFileName)
                rpt.renderToFile(intitulesXMLOutputFileName)

        def createGrantsXML(self, tableSchemaFileName):
                '''create XML schema input for grants'''
                macroFileName = self.getFileName("xmlMacros")
                templateFileName = self.getFileName("grantsXML")
                rpt = renderPageTemplate()
                rpt.setTemplateFromFile(templateFileName)
                rpt.addXMLObjectToContext(tableSchemaFileName, {'table':XMLMixin})
                rpt.addPageTemplateToContext('macro_pt', macroFileName)
                path, file = os.path.split(tableSchemaFileName)
                XMLOutputFileName = path + os.sep + rpt.context['table'].name.encode('ascii') + '_grants.xml'
                rpt.renderToFile(XMLOutputFileName)
                return XMLOutputFileName

        def includeGrants(self, tableSchemaFileName, XMLOutputFileName):
                rpt = renderPageTemplate()
                templateFileName = self.getFileName("includeGrantsXML")
                rpt.setTemplateFromFile(templateFileName)
                macroFileName =  self.getFileName("xmlMacros")
                rpt.addPageTemplateToContext('macro_pt', macroFileName)
                rpt.addXMLObjectToContext(tableSchemaFileName, {'table':TableMixin})
                grantSchemaFileName = self.createGrantsXML(tableSchemaFileName)
                rpt.addXMLObjectToContext(grantSchemaFileName, {'grant':XMLMixin})
                rpt.renderToFile(XMLOutputFileName)

        def getNomenNames(self):
                '''return sequence of nomenclature names
                '''
                result = []
                if hasattr(self.schema, 'nomenclatures') and hasattr(
                        self.schema.nomenclatures, 'nomenclature'):
                        for nomen in self.schema.nomenclatures.nomenclature:
                                result.append(nomen.PCDATA.encode('ascii'))
                return result

        def getNomenTableNames(self):
                '''return sequence of nomenclature table names
                '''
                result = []
                for nomen in self.getNomenNames():
                        result.append(nomen + '_CODES')
                        result.append(nomen + '_INTITULES')
                return result

        def getTableNamesWithoutNomens(self):
                '''return sequence of (not nomenclature) table names
                '''
                result = []
                if hasattr(self.schema, 'tables') and hasattr(self.schema.tables, 'table'):
                        for table in self.schema.tables.table:
                                tableName = table.name.encode('ascii')
                                result.append(tableName)
                return result

        def getAllTablesNames(self):
                '''return sequence of table names
                '''
                result = self.getNomenTableNames()
                return result + self.getTableNamesWithoutNomens()

        def getSourceTableSchemaFiles(self):
                result = []
                for table in self.schema.tables.table:
                        tableName = table.name.encode('ascii')
                        tableFileId = table.fileId.encode('ascii')
                        tableSchemaFileName = self.getFileName(tableFileId)
                        result.append(tableSchemaFileName)
                return ' '.join(result)

        def createSQL(self, schemaFileName, templateFileName, SQLOutputFileName):
                '''output SQL file from xml schema file and template
                '''
                dir, outputFile = os.path.split(SQLOutputFileName)
                print 'create SQL %s ' % outputFile
                rpt = renderPageTemplate()
                rpt.setTemplateFromFile(templateFileName)
                macroFileName =  self.getFileName("sqlMacros")
                rpt.addPageTemplateToContext('macro_pt', macroFileName)
                rpt.addXMLObjectToContext(schemaFileName, {'table':TableMixin})
                rpt.renderToFile(SQLOutputFileName)

        def createFromBaseSQL(self, sourceFileId, destinationFileId):
                '''output SQL file from input file and database filename, user, password
                '''
                outputCreateDbSqlFileName = self.getFileName(destinationFileId)
                createDbSqlFileName = self.getFileName(sourceFileId)
                creationSQL = open(createDbSqlFileName).read()
                creationSQLFile = open(outputCreateDbSqlFileName, "w")
                filename = self.schema.tools.database.filename.PCDATA.encode('ascii')
                user = self.schema.tools.database.user.PCDATA.encode('ascii')
                password = self.schema.tools.database.password.PCDATA.encode('ascii')
                creationSQLFile.write(creationSQL %{'fileName':filename, 'user':user, 'password':password})
                creationSQLFile.close()

        def createZopeInterface(self):
                import generateZope
                zo = generateZope.ZopeDatabaseManager(self)
                if hasattr(self.schema.nomenclatures, 'nomenclature'):
                        for nomen in self.schema.nomenclatures.nomenclature:
                                nomenName = nomen.PCDATA.encode('ascii').lower()
                                print 'nomen : %s' % nomenName
                                zo.addTable('%s_codes' % nomenName)
                                zo.addTable('%s_intitules' % nomenName)

                for table in self.schema.tables.table:
                        tableName = table.name.encode('ascii').lower()
                        zo.addTable(tableName)




if __name__ == '__main__':
        options.addOption('g', 'createGrantsXML')
        options.process()
        dg = DatabaseGeneration()
        if options.createGrantsXML:
                templateFileName = options.commandLineParams[0]
                macroFileName = options.commandLineParams[1]
                XMLOutputFileName = options.commandLineParams[2]
                tableName = options.commandLineParams[3]
                dg.createGrantsXML(templateFileName, macroFileName, XMLOutputFileName, tableName)

