from god.options import options

from renderPageTemplate import renderPageTemplate, cached_XML_Objectify

from Mixin import TableMixin
from activityMixin import ActivityMixin
from Mixin import XMLMixin

from time import *
import os

class DatabaseGeneration:
        def __init__(self, xmlConfigFileName):
                if xmlConfigFileName:
                        self.initialize(xmlConfigFileName)

        def initialize(self, xmlConfigFileName):
                self.schema = cached_XML_Objectify(xmlConfigFileName).make_instance()
                self.sql_connect = self.schema.tools.database.sql_connect.PCDATA.encode('ascii')
                self.zope_url = self.schema.tools.zope.host.url.encode('ascii')
                self.zope_db_connection = self.schema.tools.zope.database_connection.id.encode('ascii')
                self.directoryPaths = {}
                for directory in self.schema.directorys.directory:
                        self.directoryPaths[directory.id.encode('ascii')]=directory.path.encode('ascii')
                self.fileNames = {}
                for file in self.schema.files.file:
                        directoryId = file.directoryId.encode('ascii')
                        fileName = file.name.encode('ascii')
                        fullPathName = self.createFileName(directoryId, fileName)
                        fileId = file.id.encode('ascii')
                        if file.type.encode('ascii') == 'input':
                                if not os.path.exists(fullPathName):
                                        raise 'file id[%s] name[%s] does not exists !' % (fileId, fullPathName)
                        self.fileNames[fileId]= fullPathName
                                

        def main(self):
                self.createIndividualFiles()
                self.createUniqueSQL()

        def createIndividualFiles(self):
                if hasattr(self.schema.nomenclatures, 'nomenclature'):
                        for nomen in self.schema.nomenclatures.nomenclature:
                                nomenName = nomen.PCDATA.encode('ascii')
                                codesSchemaFileName, intitulesSchemaFileName = self.createNomenXML(nomenName)
                                tableName = '%s_CODES' % nomenName
                                self.createGrantsXML(tableName)
                                xmlSchemaFileName = self.createXML(codesSchemaFileName, tableName, 'full_table')
                                self.createSQLsForTable(xmlSchemaFileName, tableName)
                                tableName = '%s_INTITULES' % nomenName
                                self.createGrantsXML(tableName)
                                xmlSchemaFileName = self.createXML(intitulesSchemaFileName, tableName, 'full_table')
                                self.createSQLsForTable(xmlSchemaFileName, tableName)

                for table in self.schema.tables.table:
                        tableName = table.name.encode('ascii')
                        tableFileId = table.fileId.encode('ascii')
                        tableSchemaFileName = self.getFileName(tableFileId)
                        self.createGrantsXML(tableName)
                        xmlSchemaFileName = self.createXML(tableSchemaFileName, tableName, 'full_table')
                        self.createSQLsForTable(xmlSchemaFileName, tableName)

        def createZopeInterface(self):
                import generateZope
                zo = generateZope.ZopeDatabaseManager(self)
                if hasattr(self.schema.nomenclatures, 'nomenclature'):
                        for nomen in self.schema.nomenclatures.nomenclature:
                                nomenName = nomen.PCDATA.encode('ascii').lower()
                                print 'nomen : %s' % nomenName
                                zo.addTable('%s_codes' % nomenName)
                                zo.addTable('%s_intitules' % nomenName)

                for table in self.schema.tables.table:
                        tableName = table.name.encode('ascii').lower()
                        zo.addTable(tableName)

        def getFileName(self, fileId):
                try:
                        return self.fileNames[fileId]
                except KeyError:
                        raise 'File with Id "%s" not defined !' % fileId

        def getDirectoryPath(self, directoryId):
                try:
                        return self.createDirectoryPath(\
                        self.directoryPaths[directoryId])
                except KeyError:
                        raise 'Directory with Id "%s" not defined !' % directoryId

        def createDirectoryPath(self, directoryPath):
                if os.path.isabs(directoryPath):
                        result = directoryPath
                else:
                        result = self.getRootDirectory() + \
                                     os.sep + directoryPath
                if not os.path.exists(result):
                        os.makedirs(result)
                return result

        def getRootDirectory(self):
                try:
                        return self.directoryPaths['root']
                except KeyError:
                        return os.getcwd()

        def createFileName(self, directoryId, fileName):
                return self.getDirectoryPath(directoryId) \
                                 + os.sep + fileName

        def createUniqueSQL(self):
                uniqueFileName = self.getFileName('outputUnique')
                uf = open(uniqueFileName, 'w')
                uf.write(self.sql_connect)
                if hasattr(self.schema.nomenclatures, 'nomenclature'):
                        for nomen in self.schema.nomenclatures.nomenclature:
                                nomenName = nomen.PCDATA.encode('ascii')
                                
                                tableName = '%s_CODES' % nomenName
                                # create CODES table
                                fileRootName = 'create_table'
                                fileName = self.createFileName('gen_sql', \
                                        "%s_%s.sql" % (tableName, fileRootName))
                                uf.write(open(fileName).read())
                                # grant CODES table
                                fileRootName = 'create_grant'
                                fileName = self.createFileName('gen_sql', \
                                        "%s_%s.sql" % (tableName, fileRootName))
                                uf.write(open(fileName).read())
                                

                                tableName = '%s_INTITULES' % nomenName
                                # create INTITULES table
                                fileRootName = 'create_table'
                                fileName = self.createFileName('gen_sql', \
                                        "%s_%s.sql" % (tableName, fileRootName))
                                uf.write(open(fileName).read())
                                # grant INTITULES table
                                fileRootName = 'create_grant'
                                fileName = self.createFileName('gen_sql', \
                                        "%s_%s.sql" % (tableName, fileRootName))
                                uf.write(open(fileName).read())

                for table in self.schema.tables.table:
                        tableName = table.name.encode('ascii')
                        # create table
                        fileRootName = 'create_table'
                        fileName = self.createFileName('gen_sql', \
                                "%s_%s.sql" % (tableName, fileRootName))
                        uf.write(open(fileName).read())
                        # grant table
                        fileRootName = 'create_grant'
                        fileName = self.createFileName('gen_sql', \
                                "%s_%s.sql" % (tableName, fileRootName))
                        uf.write(open(fileName).read())
                uf.close()

        def createSQLsForTable(self, xmlSchemaFileName, tableName):
                for sql in self.schema.sqls.sql:
                        directoryId = sql.directoryId.encode('ascii')
                        fileRootName = sql.file.rootName.encode('ascii')
                        self.createSQL(xmlSchemaFileName, tableName, directoryId, fileRootName)

        def createDataSQL(self, directoryName, dataFileRootName):
                rpt = renderPageTemplate()
                templateFileName = self.getFileName('insertData')
                rpt.setTemplateFromFile(templateFileName)
                rpt.addToContext({'codeopName':'manual'})
                rpt.addToContext({'timestamp':strftime('%Y-%m-%d %H:%M:%S', localtime())})
                rpt.addToContext({'userName':'gotcha'})
                xmlFileName = directoryName + os.sep + dataFileRootName + '.xml'
                cachedXMLObject = rpt.addXMLObjectToContext(xmlFileName, {'dvd':None})
                macroFileName = self.getFileName('sqlMacros')
                rpt.addPageTemplateToContext('macro_pt', macroFileName)
                sqlFileName = self.createFileName('gen_sql', dataFileRootName + '.sql')
                rpt.renderToFile(sqlFileName)
                cachedXMLObject.wipe_instance(xmlFileName)

        def createDataSQLFomMultipleItemsInOneFile(self, xmlFileName):
                rpt = renderPageTemplate()
                templateFileName = self.getFileName('insertData')
                rpt.setTemplateFromFile(templateFileName)
                rpt.addToContext({'codeopName':'manual'})
                rpt.addToContext({'timestamp':strftime('%Y-%m-%d %H:%M:%S', localtime())})
                rpt.addToContext({'userName':'gotcha'})
                cachedXMLObject = rpt.addXMLObjectToContext(xmlFileName, {'activite':ActivityMixin})
                macroFileName = self.getFileName('sqlMacros')
                rpt.addPageTemplateToContext('macro_pt', macroFileName)
                sqlFileName = self.createFileName('gen_sql', 'data.sql')
                rpt.renderToFile(sqlFileName)
                cachedXMLObject.wipe_instance(xmlFileName)

        def createAllDataSQL(self, directoryName):
                uf = open(self.getFileName('outputUniqueData'), 'w')
                uf.write(self.sql_connect)
                print ' this should be it ', directoryName
                for fileName in os.listdir(directoryName):
                        fileRootName = fileName[:4]
                        print fileRootName
                        self.createDataSQL(directoryName, fileRootName)
                        dataFileName = self.createFileName('gen_sql', \
                                           "%s.sql" % fileRootName)
                        uf.write(open(dataFileName).read())
                uf.close()

        def createNomenXML(self, nomenName):
                #_CODES
                #create XML schema input for table
                nomenName = nomenName.upper()
                tableName = "%s_CODES" % nomenName
                codesXMLOutputFileName = self.createFileName("gen_xml", "%s.xml" % tableName)
                rpt = renderPageTemplate()
                templateFileName = self.getFileName('nomenCodes')
                rpt.setTemplateFromFile(templateFileName)
                rpt.addToContext({'nomenName':nomenName})
                macroFileName = self.getFileName('xmlMacros')
                rpt.addPageTemplateToContext('macro_pt', macroFileName)
                rpt.renderToFile(codesXMLOutputFileName)

                #_INTITULES        
                #create XML schema input for table
                tableName = "%s_INTITULES" % nomenName.upper()
                intitulesXMLOutputFileName = self.createFileName("gen_xml", "%s.xml" % tableName)
                rpt = renderPageTemplate()
                templateFileName = self.getFileName('nomenIntitules')
                rpt.setTemplateFromFile(templateFileName)
                rpt.addToContext({'nomenName':nomenName})
                macroFileName = self.getFileName('xmlMacros')
                rpt.addPageTemplateToContext('macro_pt', macroFileName)
                rpt.renderToFile(intitulesXMLOutputFileName)
                return (codesXMLOutputFileName, intitulesXMLOutputFileName)
                
        def createGrantsXML(self, tableName):
                '''create XML schema input for grants'''
                tableName = tableName.upper()
                grantsName = "%s_GRANTS" % tableName
                XMLOutputFileName = self.createFileName("gen_xml", "%s.xml" % grantsName)
                rpt = renderPageTemplate()
                templateFileName = self.getFileName('grantsXML')
                rpt.setTemplateFromFile(templateFileName)
                rpt.addToContext({'tableName':tableName})
                macroFileName = self.getFileName('xmlMacros')
                rpt.addPageTemplateToContext('macro_pt', macroFileName)
                rpt.renderToFile(XMLOutputFileName)

        def createProceduresXML(self, procedureName):
                #create XML schema input for procedures
                procedureName = procedureName.upper()
                proceduresName = "%s_PROCEDURES" % procedureName
                XMLOutputFileName = "gen_xml\\%s.xml" % proceduresName
                rpt = renderPageTemplate()
                print rpt
                rpt.setTemplateFromFile("pt\\xml\\procedures.pt")
                rpt.addToContext({'procedureName':procedureName})
                rpt.addPageTemplateToContext('macro_pt', "pt\\xml\\xml_macros.pt")
                rpt.renderToFile(XMLOutputFileName)

        def createSQLProcedures(self, procedureName, fileRootName):
                schemaFileName = "gen_xml\\%s_procedures.xml" % procedureName
                SQLOutputFileName = "gen_sql\\%s_%s.sql" % (procedureName, fileRootName)
                rpt = renderPageTemplate()
                print rpt
                print fileRootName
                rpt.setTemplateFromFile("pt\\sql\\%s.pt" % fileRootName)
                rpt.addPageTemplateToContext('macro_pt', "pt\\sql\\sql_macros.pt")
                rpt.addXMLObjectToContext(schemaFileName, {'table':TableMixin})
                rpt.renderToFile(SQLOutputFileName)        

        def createSQL(self, schemaFileName, tableName, directoryId, fileRootName):
                print 'create SQL %s for %s' % (fileRootName, tableName)
                SQLOutputFileName = self.createFileName("gen_sql", \
                                "%s_%s.sql" % (tableName, fileRootName))
                rpt = renderPageTemplate()
                templateFileName = self.createFileName(directoryId, fileRootName + ".pt")
                rpt.setTemplateFromFile(templateFileName)
                macroFileName = self.getFileName('sqlMacros')
                rpt.addPageTemplateToContext('macro_pt', macroFileName)
                rpt.addXMLObjectToContext(schemaFileName, {'table':TableMixin})
                rpt.renderToFile(SQLOutputFileName)

        def createXML(self, tableSchemaFileName, tableName, fileRootName):
                grantSchemaFileName = self.createFileName("gen_xml", "%s_grants.xml" % tableName)
                XMLOutputFileName = self.createFileName("gen_xml", \
                                "%s_%s.xml" % (tableName, fileRootName))
                rpt = renderPageTemplate()
                templateFileName = self.createFileName("common_pt_xml", fileRootName + ".pt")
                rpt.setTemplateFromFile(templateFileName)
                xmlTracingFileName = self.getFileName("tracingXML")
                rpt.addToContext({'tracing':open(xmlTracingFileName).read()})
                macroFileName = self.getFileName('xmlMacros')
                rpt.addPageTemplateToContext('macro_pt', macroFileName)
                rpt.addXMLObjectToContext(tableSchemaFileName, {'table':XMLMixin})
                rpt.addXMLObjectToContext(grantSchemaFileName, {'grant':XMLMixin})
                rpt.renderToFile(XMLOutputFileName)
                return XMLOutputFileName

if __name__ == '__main__':
        def createTableSQL(tableName):
                createGrantsXML(tableName)
                createXML(tableName, 'generated')
                createSQL(tableName, 'sql', 'create_table')

        def createBrowseSQL(tableName):
                createSQL(tableName, 'zope\\ZSQLMethod', 'zope_browse')

        def createSelectSQL(tableName):
                createSQL(tableName, 'zope_select')

        def createDeleteSQL(tableName):
                createSQL(tableName, 'zope_delete')

        def createInsertSQL(tableName):
                createSQL(tableName, 'zope_insert')
                
        def createSearchSQL(tableName):
                createSQL(tableName, 'zope_search')

        def createUpdateSQL(tableName):
                createSQL(tableName, 'zope_update')
                
        def createTrial(procedureName): 
                createProceduresXML(procedureName)
                createSQLProcedures(procedureName, 'create_procedures')


        options.addOption('c', 'create_table')
        options.addOption('t', 'trial')
        options.addOption('n', 'nomen')
        options.addOption('g', 'grant')
        options.addOption('b', 'browse')
        options.addOption('d', 'delete')
        options.addOption('data', 'data')
        options.addOption('alldata', 'alldata')
        options.addOption('i', 'insert')
        options.addOption('s', 'select')
        options.addOption('h', 'search')
        options.addOption('u', 'update')
        options.addOption('z', 'zope')

        options.process()
        dg = DatabaseGeneration(options.commandLineParams[0])
        if options.create_table:
                createTableSQL(options.commandLineParams[0])
        elif options.nomen:
                createNomenXML(options.commandLineParams[0])
        elif options.grant:
                createGrantsXML(options.commandLineParams[0])
        elif options.browse:
                createBrowseSQL(options.commandLineParams[0])
        elif options.data:
                createDataSQL('xml\\data', '4573')
        elif options.alldata:
                dg.createDataSQLFomMultipleItemsInOneFile(options.commandLineParams[1])
        elif options.delete:
                createDeleteSQL(options.commandLineParams[0])
        elif options.insert:
                createInsertSQL(options.commandLineParams[0])
        elif options.select:
                createSelectSQL(options.commandLineParams[0])
        elif options.search:
                createSearchSQL(options.commandLineParams[0])
        elif options.update:
                createUpdateSQL(options.commandLineParams[0])
        elif options.trial:
                createTrial(options.commandLineParams[0])
        elif options.zope:
                dg.createZopeInterface()
        else:
                dg.main()
