from renderPageTemplate import renderPageTemplate, cached_XML_Objectify

from god.options import options

import basicAuthXmlrpclib
import xmlrpclib

from Mixin import TableMixin

import os

cache_time = 60


import exceptions

class zopeObjectsError(exceptions.Exception):
    def __init__(self, args=None):
        self.args = args

class zopeObject:
    def __init__(self, container, path, xmlrpcFacade):
        self._container = container
        self._path = path
        self._xmlrpcFacade = xmlrpcFacade

    def __getattr__(self, attrName):
        return getattr(self._xmlrpcFacade, attrName)

    def deleteObject(self, objectId):
        try:
            objectPath = self._path + '/' + objectId
            self._container.getObject(objectPath)
            try:
                self.manage_delObjects([objectId])
            except xmlrpclib.Fault:
                pass
            self._container.clearCache(objectPath)
        except zopeObjectsError:
            pass

    def getObject(self, path):
        return self._container.getObject(self._path + '/' + path)

class zopeObjects:
    def __init__(self, baseURL):
        self.objectCache = {}
        self.baseURL = baseURL
        #self.getObject('')

    def clearCache(self, objectPath):
        del self.objectCache[objectPath]

    def getObject(self, path):
        if not self.objectCache.has_key(path):
            obj = xmlrpclib.Server(self.baseURL + path)
            try:
                obj.getId()
            except xmlrpclib.Fault:
                raise zopeObjectsError, 'no zope object of path %s' % path #return None
            self.objectCache[path] = zopeObject(self, path, obj)
        return self.objectCache[path]

    def addPythonScript(self, zopeFolder, scriptName):
        zopeFolder.deleteObject(scriptName)
        try:
            zopeFolder.manage_addProduct.PythonScripts.manage_addPythonScript(scriptName)
        except xmlrpclib.ProtocolError:
            pass
        return zopeFolder.getObject(scriptName)

    def addFormulatorForm(self, zopeFolder, formName):
        zopeFolder.deleteObject(formName)
        print 'Zope - Form : %s' % formName
        try:
            zopeFolder.manage_addProduct.Formulator.manage_add(formName)
        except xmlrpclib.ProtocolError:
            pass
        return zopeFolder.getObject(formName)

    def addAbracadabraObject(self, zopeFolder, id, object_id, title=''):
        zopeFolder.deleteObject(id)
        print 'Zope - AbracadabraObject : %s' % id
        try:
            zopeFolder.manage_addProduct.AbracadabraObject.manage_addAbracadabraObject(id, object_id, title)
        except xmlrpclib.ProtocolError:
            pass
        return zopeFolder.getObject(id)

    def addZSQLMethod(self, zopeFolder, methodName):
        zopeFolder.deleteObject(methodName)
        print 'Zope - ZSQL : %s' % methodName
        try:
            zopeFolder.manage_addProduct.ZSQLMethods.manage_addZSQLMethod(methodName, '', '', '', '')
        except xmlrpclib.ProtocolError:
            pass
        return zopeFolder.getObject(methodName)

    def addFolders(self, relativePath):
        folders = relativePath.split('/')
        folder = self
        for folderName in folders:
            folder = self.addFolder(folder, folderName)
        return folder

    def addFolder(self, zopeFolder, folderName):
        try:
            return zopeFolder.getObject(folderName)
        except zopeObjectsError:
            print 'Zope - Folder : %s' % folderName
            zopeFolder.manage_addFolder(folderName, '')
            return zopeFolder.getObject(folderName)

    def addEditPythonScript(self, folder, scriptName, code):
        print 'Zope - Python Script : %s' % scriptName
        script = self.addPythonScript(folder, scriptName)
        script.write(code)
        return script

    def addFormulatorField(self, form, id, title, fieldType):
        form.deleteObject(id)
        print 'Zope - Form Field : %s' % id
        try:
            form.manage_addProduct.Formulator.manage_addField(id, title, fieldType)
        except xmlrpclib.ProtocolError:
            pass
        return form.getObject(id)

    def editProperties(self, instance, propertyMap):
            propertyIds = instance.propertyIds()
            changedProperties = {}
            for id, value, type in propertyMap.values():
                    if id in propertyIds:
                        changedProperties[id] = value
                    else:
                        instance.manage_addProperty(id, value, type)
            if changedProperties:
                instance.manage_changeProperties(changedProperties)

class ZopeDatabaseManager(zopeObjects):
    def __init__(self, databaseGenerator):
            self.dg = databaseGenerator
            zopeObjects.__init__(self, self.dg.zope_url)
            self.dbConnectionId = self.dg.zope_db_connection

    def getTablesFolder(self):
        if not hasattr(self, '_tablesFolder'):
            self._tablesFolder = self.getObject('tables')
        return self._tablesFolder

    def genXMLDirectory(self):
        return self.dg.getDirectoryPath('gen_xml') + os.sep

    def genSQLDirectory(self):
        return self.dg.getDirectoryPath('gen_sql') + os.sep

    def ZSQLMethodDirectory(self):
        return self.dg.getDirectoryPath('common_pt_ZSQL') + os.sep

    def pythonDirectory(self):
        return self.dg.getDirectoryPath('py') + os.sep

    def addField(self, formulatorForm, xmlField, required):
        formId = formulatorForm.getId()
        fieldName = xmlField.getName()
        fieldType = xmlField.getType()
        field_args = {}
        FormulatorFieldType = ''
        tales_field_args = {}
        propertyMap = {}
        lookup = xmlField.getLookup()
        if lookup:
            fkTableName = lookup.tableName
            fkResultFieldName = lookup.resultFieldName
            FormulatorFieldType = 'ListField'
            if (formId in ['insertForm', 'updateForm']):
                tales_field_args['items'] = "python:form.%s.get%sItems('update')" % (fkTableName, fkResultFieldName)
            elif formId == 'searchForm':
                tales_field_args['items'] = "python:form.%s.get%sItems('search')" % (fkTableName, fkResultFieldName)                    
            field_args['size'] = 1
            self.addGetItemsScript(lookup)
            propertyMap['fkTable']=('fkTable', fkTableName, 'string')
        if FormulatorFieldType == '':           
            if fieldType == "INTEGER":
                FormulatorFieldType = 'IntegerField'
            elif fieldType == "DECIMAL":
                FormulatorFieldType = 'FloatField'
            elif fieldType == "FLOAT":
                FormulatorFieldType = 'FloatField'
            elif fieldType == "DATE":
                FormulatorFieldType = 'DateTimeField'
                field_args['date_only'] = 1            
            elif fieldType == "TIMESTAMP":
                if formId=='searchForm':
                    FormulatorFieldType = 'StringField'
                else:
                    FormulatorFieldType = 'DateTimeField'
                    field_args['date_only'] = 0
            elif (fieldType == "CHAR") or (fieldType == "VARCHAR"):
                FormulatorFieldType = 'StringField'
        if FormulatorFieldType == 'StringField':
            if hasattr(xmlField, 'length'):
                fieldLength = int(xmlField.length)
                if fieldLength > 50:
                    FormulatorFieldType = 'TextAreaField'
                    field_args['width'] = 50
                    field_args['height'] = (fieldLength / 50) + 1
                else:
                    field_args['display_width'] = fieldLength
                    field_args['display_maxwidth'] = fieldLength
                field_args['max_length'] = fieldLength
        elif FormulatorFieldType == 'IntegerField':
            field_args['display_width'] = 6
            field_args['display_maxwidth'] = 6
        if hasattr(xmlField, 'display'):
            xmlDisplay = xmlField.display
            field_args['title'] = xmlField.getDisplayLabel()
            if hasattr(xmlDisplay, 'table'):
                visible = xmlDisplay.table.visible.encode('latin-1')
                if visible == 'true':
                    propertyMap['visible']=('visible', 1, 'boolean')
                elif visible == 'false':
                    propertyMap['visible']=('visible', 0, 'boolean')
                else:
                    raise zopeObjectsError, 'invalid visible value'
        if hasattr(xmlField, 'primarykey'):
            if (formId == 'updateForm'):
                field_args['hidden'] = 1
        zo_field = self.addFormulatorField(formulatorForm, fieldName, '', FormulatorFieldType)
        field_args['required'] = required
        zo_field.manage_edit(field_args)
        if tales_field_args:
            exec 'formulatorForm.manage_addProduct.Formulator.%s.manage_tales(tales_field_args)' % fieldName.upper()
        self.editProperties(zo_field, propertyMap)

    def addGetItemsScript(self, lookup):
        fkTableName = lookup.tableName
        fkResultFieldName = lookup.resultFieldName
        tableFolder = self.getObject('tables/%s' % fkTableName)
        scriptName = 'get%sItems' % fkResultFieldName
        if not (scriptName in tableFolder.getObjectIds()):
            ZSQLName = 'SQL_list%sItems' % fkResultFieldName 
            codeFileName = self.dg.getFileName('listItems')
            code = open(codeFileName).read() % ZSQLName
            self.addEditPythonScript(tableFolder, scriptName, code)
            # add SQL
            storageFolder = self.getObject('tables/%s/storage' % fkTableName)
            if not (ZSQLName in storageFolder.getObjectIds()):
                sql_listItems = self.addZSQLMethod(storageFolder, ZSQLName)
                args = ''
                templateFileName = '%szope_listItems.pt' % (self.ZSQLMethodDirectory())
                self.tablePT.setTemplateFromFile(templateFileName)
                self.tablePT.context = {}
                self.tablePT.addToContext({'lookup':lookup})
                SQL = self.tablePT.render()
                sql_listItems.manage_edit('', self.dbConnectionId , args, SQL)
    
    def createTableFolder(self, tableName, cfgTable):
        tableFolder = self.addFolder(self.getTablesFolder(), tableName)

        propertyMap={}
        propertyMap['title']=('title', cfgTable.getDisplayLabel(), 'string')
        propertyMap['description']=('description', cfgTable.getDisplayDescription(), 'lines')
        self.editProperties(tableFolder, propertyMap)

##        self.addEditPythonScript(tableFolder, 'beforeInsert', '')    
##
##        self.addEditPythonScript(tableFolder, 'beforeUpdate', '')

        pts = '''
<tal:bs repeat="field table/likeFields">context.logic.database.processOptionalStringSQLParameter('<tal:fieldname replace="field/name"/>')</tal:bs>
'''
        self.tablePT.setTemplateFromString(pts)
        code = self.tablePT.render()
        self.addEditPythonScript(tableFolder, 'beforeSearch', code)

        pts = '''
<tal:ps>return [<tal:primarykeys repeat="field table/primarykeys">'<tal:fieldname replace="field/name"/>'<tal:comma condition="not:repeat/field/end">, </tal:comma></tal:primarykeys>]</tal:ps>
'''
        self.tablePT.setTemplateFromString(pts)
        code = self.tablePT.render()
        getPrimaryKeyFieldNames = self.addEditPythonScript(tableFolder, 'getPrimaryKeyFieldNames', code)
        getPrimaryKeyFieldNames.ZCacheable_setManagerId('ramcache')

        return tableFolder


    def createFieldsFolder(self, tableFolder, cfgTable):
        tableFolder.deleteObject('fields')
        fieldsFolder = self.addFolder(tableFolder, 'fields')
        i = 0
        for xmlField in cfgTable.tracingExcludedFields():
            zopeField = self.addAbracadabraObject(fieldsFolder, xmlField.getName(), 'field', xmlField.getDisplayLabel())
            propertyMap = {}
            propertyMap['order']=('order', i, 'int')
            propertyMap['visible']=('visible', xmlField.getDisplayVisible(), 'boolean')
            propertyMap.update(xmlField.getFieldPropertyMap())
            self.editProperties(zopeField, propertyMap)
            i += 1
            lookup = xmlField.getLookup()
            if lookup and lookup.display:
                self.createLookupPythonScriptAndSQL(lookup, xmlField.getZSQLDTMLForFieldValueWithParam(lookup.codeFieldName))
        return fieldsFolder

    def createLookupPythonScriptAndSQL(self, lookup, dtml):
        lkTableName = lookup.tableName
        lkResultFieldName = lookup.resultFieldName
        lkCodeFieldName = lookup.codeFieldName
        map = {'code':lkCodeFieldName, 'result':lkResultFieldName, 'table':lkTableName.upper(), 'dtml':dtml}
        tableFolder = self.getObject('tables/%s' % lkTableName)
        scriptName = 'get%sBy%s' % (lkResultFieldName, lkCodeFieldName)
        if not (scriptName in tableFolder.getObjectIds()):
            codeFileName = self.dg.getFileName('listItems')
            code = '''
##parameters=%(code)s

return container.storage.SQL_get%(result)sBy%(code)s(%(code)s=%(code)s)[0][0]
'''
            lookupPythonScript = self.addEditPythonScript(tableFolder, scriptName, code % map)
            lookupPythonScript.ZCacheable_setManagerId('ramcache')
            storageFolder = self.getObject('tables/%s/storage' % lkTableName)
            ZSQLName = 'SQL_get%sBy%s' % (lkResultFieldName, lkCodeFieldName)
            if not (ZSQLName in storageFolder.getObjectIds()):
                sql_listItems = self.addZSQLMethod(storageFolder, ZSQLName)
                args = lkCodeFieldName
                SQL = '''
select %(result)s from %(table)s
where (%(code)s=%(dtml)s)
'''
                sql_listItems.manage_edit('', self.dbConnectionId , args, SQL % map)
                sql_listItems.manage_advanced(1000, 100, cache_time, '', '')


    def addTable(self, tableName):
        print 'Zope table folder : %s' % tableName
        self.tablePT = renderPageTemplate()
        self.tablePT.addXMLObjectToContext('%s%s_full_table.xml' % (self.genXMLDirectory(), tableName), {'table':TableMixin})
        cfgTable = self.tablePT.context['table']


        tableFolder = self.createTableFolder(tableName, cfgTable)

        fieldsFolder = self.createFieldsFolder(tableFolder, cfgTable)


        insertForm = self.addFormulatorForm(tableFolder, 'insertForm')

        for field in cfgTable.updateableFields():
            self.addField(insertForm, field, required=hasattr(field, 'notnull'))

        updateForm = self.addFormulatorForm(tableFolder, 'updateForm')

        for field in cfgTable.tracingExcludedFields():
            self.addField(updateForm, field, required=hasattr(field, 'notnull'))

        searchForm = self.addFormulatorForm(tableFolder, 'searchForm')

        for field in cfgTable.tracingExcludedFields():
            self.addField(searchForm, field, required=0)

    def addStorageZsqlBrowse(self, tableName):
        storageFolder = self.addFolders('tables/%s/storage' % tableName)
        
        sql_browse = self.addZSQLMethod(storageFolder, 'SQL_browse')
        args = ''
        SQLFileName = '%s%s_zope_browse.sql' % (self.genSQLDirectory(), tableName)
        SQL = open(SQLFileName).read()
        sql_browse.manage_edit('', self.dbConnectionId , args, SQL)
        sql_browse.manage_advanced(1000, 100, 0, 'formattedRecord', 'formattedRecord.py')

    def addStorageZsqlSearch(self, tableName):
        storageFolder = self.addFolders('tables/%s/storage' % tableName)
        
        sql_search = self.addZSQLMethod(storageFolder, 'SQL_search')
        argsFileName = '%s%s_zope_searchablefields_args.sql' % (self.genSQLDirectory(), tableName)
        args = open(argsFileName).read()
        SQLFileName = '%s%s_zope_search.sql' % (self.genSQLDirectory(), tableName)
        SQL = open(SQLFileName).read()
        sql_search.manage_edit('', self.dbConnectionId , args, SQL)
        sql_search.manage_advanced(1000, 100, 0, 'formattedRecord', 'formattedRecord.py')

    def addStorageZsqlDelete(self, tableName):
        storageFolder = self.addFolders('tables/%s/storage' % tableName)
        
        sql_delete = self.addZSQLMethod(storageFolder, 'SQL_delete')
        argsFileName = '%s%s_zope_primarykeys_args.sql' % (self.genSQLDirectory(), tableName)
        args = open(argsFileName).read()
        SQLFileName = '%s%s_zope_delete.sql' % (self.genSQLDirectory(), tableName)
        SQL = open(SQLFileName).read()
        sql_delete.manage_edit('', self.dbConnectionId , args, SQL)

    def addStorageZsqlInsert(self, tableName):
        storageFolder = self.addFolders('tables/%s/storage' % tableName)
        
        sql_insert = self.addZSQLMethod(storageFolder, 'SQL_insert')
        argsFileName = '%s%s_zope_nogeneratorfields_args.sql' % (self.genSQLDirectory(), tableName)
        args = open(argsFileName).read()
        SQLFileName = '%s%s_zope_insert.sql' % (self.genSQLDirectory(), tableName)
        SQL = open(SQLFileName).read()
        sql_insert.manage_edit('', self.dbConnectionId , args, SQL)

    def addStorageZsqlUpdate(self, tableName):
        storageFolder = self.addFolders('tables/%s/storage' % tableName)
        
        sql_update = self.addZSQLMethod(storageFolder, 'SQL_update')
        argsFileName = '%s%s_zope_allfields_args.sql' % (self.genSQLDirectory(), tableName)
        args = open(argsFileName).read()
        SQLFileName = '%s%s_zope_update.sql' % (self.genSQLDirectory(), tableName)
        SQL = open(SQLFileName).read()
        sql_update.manage_edit('', self.dbConnectionId , args, SQL)

    def addStorageZsqlSelect(self, tableName):
        storageFolder = self.addFolders('tables/%s/storage' % tableName)
        
        sql_select = self.addZSQLMethod(storageFolder, 'SQL_primaryKeySelect')
        argsFileName = '%s%s_zope_primarykeys_args.sql' % (self.genSQLDirectory(), tableName)
        args = open(argsFileName).read()
        SQLFileName = '%s%s_zope_select.sql' % (self.genSQLDirectory(), tableName)
        SQL = open(SQLFileName).read()
        sql_select.manage_edit('', self.dbConnectionId , args, SQL)
        sql_select.manage_advanced(1000, 100, 0, '', '')

def test():
    pass

def main(tableName):
    import generateDatabase
    gb = generateDatabase.DatabaseGeneration(r'e:\code\python\generatedatabase\projects\boussole\xml\database.xml')
    zo = ZopeDatabaseManager(gb)
    zo.addTable(tableName)
    

if __name__ == '__main__':
    options.addOption('t', 'test')
    options.process()
    if options.test:
        test()
    else:
        main(options.commandLineParams[0])


 
