SET SQL DIALECT 3;
/* Create database :   */
CREATE DATABASE "tests/interbase/tests.gdb" USER "SYSDBA" PASSWORD "celodor" PAGE_SIZE 1024 ;

/***********************************/
/*              Tables             */
/***********************************/


/* Table: CODES_LANGUE_SITE  */
CREATE TABLE CODES_LANGUE_SITE (
       CODE CHAR(3) NOT NULL,
       DESCRIPTION VARCHAR(100),
      PRIMARY KEY (CODE)
);

/* Table: CODES_OPERATION  */
CREATE TABLE CODES_OPERATION (
       CODE CHAR(15) NOT NULL,
       TS TIMESTAMP NOT NULL,
       DESCRIPTION VARCHAR(250) NOT NULL,
      PRIMARY KEY (CODE)
);

/* Table: CODES_PERSONNE_EQUIPE  */
CREATE TABLE CODES_PERSONNE_EQUIPE (
       CODE CHAR(3) NOT NULL,
       ZOPEUSERID VARCHAR(10) NOT NULL,
       DESCRIPTION VARCHAR(100),
      PRIMARY KEY (CODE)
);


/*---------------------------*/
/*  Foreign Key Definitions  */
/*---------------------------*/

CREATE GENERATOR CODE_OP_GEN;

SET GENERATOR CODE_OP_GEN TO 0;

/***********************************/
/*             Exceptions          */
/***********************************/

CREATE EXCEPTION UNAUTHENTIFIED_ZOPE_USER 'This Zope user is not known in table CODES_PERSONNE_EQUIPE !';

COMMIT WORK;
SET TERM ^ ;
/***********************************/
/*        Stored Procedures        */
/***********************************/

CREATE PROCEDURE CHECK_CODE_OPERATION(I_CODE VARCHAR(250),I_TS TIMESTAMP) RETURNS (O_CODE VARCHAR(250)) AS BEGIN EXIT; END ^
CREATE PROCEDURE CHECK_PERSONNE_EQUIPE(I_ZOPEUSERID VARCHAR(10)) RETURNS (O_CODE CHAR(3)) AS BEGIN EXIT; END ^
CREATE PROCEDURE COMMON_TRACING(I_CODE_OP VARCHAR(250),I_QUI VARCHAR(10),I_TS TIMESTAMP) RETURNS (O_CODE_OP VARCHAR(250),O_QUI VARCHAR(10),O_TS TIMESTAMP) AS BEGIN EXIT; END ^
CREATE PROCEDURE CONVERT_INTEGER_TO_CHAR15(I_INT INTEGER) RETURNS (O_CHAR VARCHAR(15)) AS BEGIN EXIT; END ^
CREATE PROCEDURE LEN_STR(I_STR VARCHAR(100)) RETURNS (O_LEN INTEGER) AS BEGIN EXIT; END ^

/* Procedure: CHECK_CODE_OPERATION  */
ALTER PROCEDURE CHECK_CODE_OPERATION(I_CODE VARCHAR(250),I_TS TIMESTAMP)
RETURNS (O_CODE VARCHAR(250))
AS
     declare variable l_code char(15);
begin
select code from codes_operation where description = :i_code and ts = :i_ts into :o_code;
if (o_code IS NULL) then
  begin
  l_code = '  ';
  insert into codes_operation (code, ts, description) values (:l_code, :i_ts, :i_code);
  select code from codes_operation where description = :i_code and ts = :i_ts into :o_code;
  end
end
 ^ 

/* Procedure: CHECK_PERSONNE_EQUIPE  */
ALTER PROCEDURE CHECK_PERSONNE_EQUIPE(I_ZOPEUSERID VARCHAR(10))
RETURNS (O_CODE CHAR(3))
AS
begin
select CODE from codes_personne_equipe where ZOPEUSERID = :i_zopeuserid into :o_code;
if (o_code IS NULL) then
  begin
  EXCEPTION UNAUTHENTIFIED_ZOPE_USER;
  end
end
 ^ 

/* Procedure: COMMON_TRACING  */
ALTER PROCEDURE COMMON_TRACING(I_CODE_OP VARCHAR(250),I_QUI VARCHAR(10),I_TS TIMESTAMP)
RETURNS (O_CODE_OP VARCHAR(250),O_QUI VARCHAR(10),O_TS TIMESTAMP)
AS
begin
        execute procedure check_code_operation i_code_op, i_ts returning_values o_code_op;
        execute procedure check_personne_equipe i_qui returning_values o_qui;
        o_ts = current_timestamp;
end
 ^ 

/* Procedure: CONVERT_INTEGER_TO_CHAR15  */
ALTER PROCEDURE CONVERT_INTEGER_TO_CHAR15(I_INT INTEGER)
RETURNS (O_CHAR VARCHAR(15))
AS
     declare variable l_length Integer;
begin
    o_char = cast(i_int as varchar(15));
    execute procedure len_str o_char returning_values l_length;
    WHILE (l_length < 15) DO BEGIN
          o_char = '0' || o_char;
          l_length = l_length + 1;
    END
end
 ^ 

/* Procedure: LEN_STR  */
ALTER PROCEDURE LEN_STR(I_STR VARCHAR(100))
RETURNS (O_LEN INTEGER)
AS
     declare variable l_pat Varchar(100);
begin
    o_len = null;
    IF (i_str IS NULL) THEN EXIT;
    l_pat = '';
    o_len = 0;
    WHILE (NOT i_str LIKE l_pat) DO BEGIN
          l_pat = l_pat || '_';
          o_len = o_len + 1;
    END
end
 ^ 


CREATE TRIGGER CREATE_CODE_OPERATION FOR CODES_OPERATION
ACTIVE BEFORE INSERT POSITION 0
AS
     declare variable l_code Integer;
     declare variable l_code_char varchar(15);
begin
l_code = GEN_ID (CODE_OP_GEN, 1);
execute procedure convert_integer_to_char15 l_code returning_values l_code_char;
new.code = l_code_char;

end
 ^ 
COMMIT WORK^
SET TERM ; ^

/* Grant role for this database */

CREATE ROLE MANAGER;
CREATE ROLE ENCODER;

/* Grant permissions for this database */
GRANT SELECT ON CODES_LANGUE_SITE TO DVD_ENC;
GRANT DELETE,INSERT,REFERENCES,SELECT,UPDATE ON CODES_LANGUE_SITE TO DVD_MAN;
GRANT SELECT ON CODES_OPERATION TO DVD_ENC;
GRANT INSERT,SELECT ON CODES_OPERATION TO DVD_MAN;
GRANT SELECT ON CODES_PERSONNE_EQUIPE TO DVD_ENC;
GRANT DELETE,INSERT,REFERENCES,SELECT,UPDATE ON CODES_PERSONNE_EQUIPE TO DVD_MAN;
GRANT EXECUTE ON PROCEDURE CHECK_CODE_OPERATION TO DVD_MAN;
GRANT EXECUTE ON PROCEDURE CHECK_PERSONNE_EQUIPE TO DVD_MAN;
GRANT EXECUTE ON PROCEDURE COMMON_TRACING TO DVD_MAN;
GRANT EXECUTE ON PROCEDURE CONVERT_INTEGER_TO_CHAR15 TO DVD_MAN;
GRANT EXECUTE ON PROCEDURE LEN_STR TO DVD_MAN;

COMMIT WORK;

/*  Table Data: CODES_LANGUE_SITE    */
INSERT INTO CODES_LANGUE_SITE(CODE,DESCRIPTION) VALUES('FRA','Franais');
INSERT INTO CODES_LANGUE_SITE(CODE,DESCRIPTION) VALUES('ENG','English');
INSERT INTO CODES_LANGUE_SITE(CODE,DESCRIPTION) VALUES('DEU','Deutsch');

COMMIT WORK;
/*  Table Data: CODES_OPERATION    */
/*  Table Data: CODES_PERSONNE_EQUIPE    */
INSERT INTO CODES_PERSONNE_EQUIPE(CODE,ZOPEUSERID,DESCRIPTION) VALUES('GCH','gotcha','Godefroid Chapelle');

COMMIT WORK;

